@@ -33,9 +33,6 @@ THE SOFTWARE.
 #include "kernel_socket.c"
 #endif
 
-int src_table_idx = 10;
-int src_table_prio = 100;
-
 /* Like gettimeofday, but returns monotonic time.  If POSIX clocks are not
    available, falls back to gettimeofday but enforces monotonicity. */
 int
@@ -85,20 +82,22 @@ gettime(struct timeval *tv)
    caller will deal with gracefully. */
 
 int
-read_random_bytes(void *buf, size_t len)
+read_random_bytes(void *buf, int len)
 {
-    int fd;
-    int rc;
+    int fd, rc;
 
     fd = open("/dev/urandom", O_RDONLY);
     if(fd < 0) {
-        rc = -1;
-    } else {
-        rc = read(fd, buf, len);
-        if(rc < 0 || (unsigned)rc < len)
-            rc = -1;
-        close(fd);
+        errno = ENOSYS;
+        return -1;
     }
+
+    rc = read(fd, buf, len);
+    if(rc < len)
+        rc = -1;
+
+    close(fd);
+
     return rc;
 }
 
@@ -119,12 +118,12 @@ kernel_older_than(const char *sysname, int version, int sub_version)
     int v = 0;
     int sub_v = 0;
     rc = uname(&un);
-    if (rc < 0)
+    if(rc < 0)
         return -1;
-    if (strcmp(sysname, un.sysname) != 0)
+    if(strcmp(sysname, un.sysname) != 0)
         return -1;
     rc = sscanf(un.release, "%d.%d", &v, &sub_v);
-    if (rc < 2)
+    if(rc < 2)
         return -1;
     return (v < version || (v == version && sub_v < sub_version));
 }
